/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:expandtab:shiftwidth=2:tabstop=2:
 */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef __nsGNOMEShellSearchProvider_h__
#define __nsGNOMEShellSearchProvider_h__

#include "mozilla/DBusHelpers.h"
#include "nsINavHistoryService.h"
#include "nsUnixRemoteServer.h"
#include "nsCOMPtr.h"
#include "mozilla/UniquePtr.h"

#define MAX_SEARCH_RESULTS_NUM 9

class GnomeHistoryIcon {
 public:
  GnomeHistoryIcon() : mSearchSerial(-1), mWidth(0), mHeight(0){};

  // From which search is this icon
  void Set(int aSearchSerial, mozilla::UniquePtr<uint8_t[]> aData, int aWidth,
           int aHeight) {
    mSearchSerial = aSearchSerial;
    mWidth = aWidth;
    mHeight = aHeight;
    mData = std::move(aData);
  }

  int GetSearchSerial() { return mSearchSerial; }
  void AppendIcon(DBusMessageIter* aIter);

 private:
  int mSearchSerial;
  mozilla::UniquePtr<uint8_t[]> mData;
  int mWidth;
  int mHeight;
};

class nsGNOMEShellSearchProvider : public nsUnixRemoteServer {
 public:
  nsGNOMEShellSearchProvider();
  ~nsGNOMEShellSearchProvider() { Shutdown(); }

  nsresult Startup();
  void Shutdown();

  DBusHandlerResult HandleDBusMessage(DBusConnection* aConnection,
                                      DBusMessage* msg);
  void UnregisterDBusInterface(DBusConnection* aConnection);

  int GetSearchSerial() { return mSearchSerial; }
  void SetHistoryIcon(int aSearchSerial, mozilla::UniquePtr<uint8_t[]> aData,
                      int aWidth, int aHeight, int aIconIndex);

 private:
  DBusHandlerResult Introspect(DBusMessage* msg);

  DBusHandlerResult GetInitialResultSet(DBusMessage* msg);
  DBusHandlerResult GetSubsearchResultSet(DBusMessage* msg);
  DBusHandlerResult GetResultMetas(DBusMessage* msg);
  DBusHandlerResult ActivateResult(DBusMessage* msg);
  DBusHandlerResult LaunchSearch(DBusMessage* msg);

  nsresult NewHistorySearch(const char* aSearchTerm);
  void GetIDKeyForURI(int aIndex, nsAutoCString& aUri, nsAutoCString& aIDKey);
  int GetIndexFromIDKey(const char* aIDKey);
  bool IsHistoryResultNodeURI(nsINavHistoryResultNode* aHistoryNode);
  void AppendResultID(DBusMessageIter* aIter, const char* aID);
  void AppendSearchID(DBusMessageIter* aIter, const char* aID);
  void ComposeSearchResultReply(DBusMessage* aReply, const char* aSearchTerm,
                                bool aSearchOnly);
  void LaunchWithID(const char* aID, uint32_t aTimeStamp);
  void LaunchWithAllResults(uint32_t aTimeStamp);

  // The connection is owned by DBus library
  RefPtr<DBusConnection> mConnection;
  nsCOMPtr<nsINavHistoryContainerResultNode> mHistResultContainer;
  nsCOMPtr<nsINavHistoryService> mHistoryService;
  nsAutoCStringN<32> mSearchTerm;
  nsAutoCString mGnomeSearchTitle;
  int mSearchSerial;
  GnomeHistoryIcon mHistoryIcons[MAX_SEARCH_RESULTS_NUM];
};

#endif  // __nsGNOMEShellSearchProvider_h__
